from django.urls import path

from . import views

app_name = "yarik_django_auth"
urlpatterns = [
    path("", views.index, name="index"),
    path("login/", views.login_view, name="login"),
    path("logout/", views.logout_view, name="logout"),
    path("keycloak_callback/", views.keycloak_callback, name="keycloak_callback"),
    path("csrftoken/", views.get_new_csrftoken, name="csrftoken"),
]



import datetime
import logging
from base64 import b64decode
from typing import Any
from urllib.parse import quote, urlencode

import jwt
import requests
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric.types import rsa
from django.contrib.auth.models import User

from yarik_django_auth.conf.settings import (
    CLIENT_ID,
    CLIENT_SECRET,
    DEFAULT_REQUEST_TIMEOUT,
    KEYCLOAK_AUDIENCE,
    KEYCLOAK_CERT_FILENAME,
    KEYCLOAK_IS_CREATE,
    KEYCLOAK_REDIRECT_URI,
    KEYCLOAK_SCOPES,
    KEYCLOAK_URL_BASE,
    REALM_NAME,
)

logger = logging.getLogger(__name__)


class KeycloakConfidentialBackend:
    @staticmethod
    def exchange_code_for_token(code: str) -> dict[str, Any] | None:
        """Возвращает токен пользователя."""
        token_endpoint = f"{KEYCLOAK_URL_BASE}realms/{REALM_NAME}/protocol/openid-connect/token"

        payload: dict[str, Any] = {
            "code": code,
            "grant_type": "authorization_code",
            "client_id": CLIENT_ID,
            "client_secret": CLIENT_SECRET,
            "redirect_uri": KEYCLOAK_REDIRECT_URI,
            "scope": " ".join(KEYCLOAK_SCOPES or []),
        }

        response = requests.post(
            token_endpoint,
            data=urlencode(payload, quote_via=quote),
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            verify=KEYCLOAK_CERT_FILENAME,
            timeout=DEFAULT_REQUEST_TIMEOUT,
        )

        if response.status_code == 200:
            r = response.json()
            logger.debug("Ответ токеном: %s", r)
            return r
        logger.info("Токен не получен: %s", response)
        return None

    @property
    def public_key(self) -> rsa.RSAPublicKey:
        """Возвращает публичный ключ из Keycloak."""
        r = requests.get(
            f"{KEYCLOAK_URL_BASE}realms/{REALM_NAME}/",
            verify=KEYCLOAK_CERT_FILENAME,
            timeout=DEFAULT_REQUEST_TIMEOUT,
        )
        r.raise_for_status()
        key_der_base64 = r.json()["public_key"]
        key_der = b64decode(key_der_base64.encode())
        pk = serialization.load_der_public_key(key_der)
        if not isinstance(pk, rsa.RSAPublicKey):
            msg = "Тип публичного ключа не поддерживается"
            raise TypeError(msg)

        return pk

    def decode_token(self, data: dict[str, Any]) -> dict[str, Any]:
        """Возвращает декодированные данные из токена."""
        access_token = data.get("access_token")
        if not access_token:
            msg = "access_token не найден в словаре"
            raise KeyError(msg)

        access_token = str(access_token)

        logger.debug("Текущее время: %s", datetime.datetime.now(tz=datetime.UTC))
        logger.debug("Публичный ключ: %s", self.public_key.public_numbers())

        return jwt.decode(
            access_token,
            key=self.public_key,
            algorithms=["RS256"],
            audience=KEYCLOAK_AUDIENCE,
        )

    def authenticate(self, token: dict[str, Any]) -> User | None:
        logger.debug("Токен: %s", token)
        logger.info("Декодирование токена доступа...")
        user_info = self.decode_token(token)
        logger.debug("Декодированный токен: %s", user_info)
        logger.info("Токен успешно декодирован!")
        logger.info("Проверка роли...")
        if (
            CLIENT_ID not in user_info["resource_access"]
            or "access" not in user_info["resource_access"][CLIENT_ID]["roles"]
        ):
            msg = "Не назначена роль access!"  # noqa: RUF001
            logger.debug(msg)
            raise PermissionError(msg)
        logger.info("Роль успешно проверена, доступ получен!")

        logger.info("Поиск пользователя в БД")
        user = self.get_user(user_info=user_info)

        if not user:
            logger.info("Пользователь не найден")

        return user

    def get_user(self, user_info: dict[str, Any]) -> User | None:
        logger.debug("Метаданные пользователя Keycloak: %s", user_info)
        user = User.objects.filter(username=user_info.get("preferred_username", "")).first()
        user_roles = user_info.get("resource_access", {}).get(CLIENT_ID, {}).get("roles", [])
        if KEYCLOAK_IS_CREATE:
            username = user_info.get("preferred_username")
            if not username:
                msg = "Не задано имя пользователя в токене"  # noqa: RUF001
                raise ValueError(msg)
            if not user:
                # Создание пользователя
                logger.info("Данный пользователь не зарегистрирован в локальной БД: создание пользователя")
                user = User.objects.create_user(
                    username=username,
                    email=user_info.get("email"),
                    last_name=user_info.get("given_name"),
                    first_name=user_info.get("family_name"),
                    is_superuser="superuser" in user_roles,
                    is_staff="staff" in user_roles,
                )

                # Установка пустого пароля
                user.set_unusable_password()

                # Сохранение пользователя
                user.save()
            else:
                logger.info("Данный пользователь уже зарегистрирован в локальной БД: обновление информации")
                user.email = user_info.get("email") or ""
                user.last_name = user_info.get("given_name") or ""
                user.first_name = user_info.get("family_name") or ""
                user.is_superuser = "superuser" in user_roles
                user.is_staff = "staff" in user_roles
                user.save()
        return user



from django import forms


class LoginForm(forms.Form):
    username = forms.CharField(widget=forms.TextInput(attrs={"placeholder": "Username", "class": "form-control"}))
    password = forms.CharField(widget=forms.PasswordInput(attrs={"placeholder": "Password", "class": "form-control"}))



import logging

import jwt
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.http import (
    HttpRequest,
    HttpResponse,
    HttpResponsePermanentRedirect,
    HttpResponseRedirect,
    JsonResponse,
)
from django.middleware.csrf import get_token
from django.shortcuts import redirect, render
from django.urls import reverse_lazy

from yarik_django_auth.conf.settings import (
    AUTH_PAGE_DOCUMENT,
    CACHE_CALLBACK,
    CLEAR_CACHE_CALLBACK,
    CLIENT_ID,
    DEFAULT_REDIRECT_URI,
    ERROR_PAGE_DOCUMENT,
    KEYCLOAK_POST_LOGOUT_URL,
    KEYCLOAK_REDIRECT_URI,
    KEYCLOAK_SCOPES,
    KEYCLOAK_URL_BASE_CLIENT,
    REALM_NAME,
    USE_KEYCLOAK,
    VERSION_TAG,
)
from yarik_django_auth.utils import ajax_login_required, clear_previos_sessions

from .backends import KeycloakConfidentialBackend
from .forms import LoginForm

logger = logging.getLogger(__name__)


def index(_: HttpRequest) -> HttpResponseRedirect | HttpResponsePermanentRedirect:
    return redirect(reverse_lazy("yarik_django_auth:login"))


def login_view(
    request: HttpRequest,
) -> HttpResponseRedirect | HttpResponsePermanentRedirect | HttpResponse:
    if USE_KEYCLOAK:
        if request.user and request.user.is_authenticated:
            return redirect(request.GET.get("next", DEFAULT_REDIRECT_URI))
        redirect_url = (
            f"{KEYCLOAK_URL_BASE_CLIENT}realms/{REALM_NAME}/protocol/openid-connect/auth"
            f"?client_id={CLIENT_ID}&response_type=code&scope={'%20'.join(KEYCLOAK_SCOPES or [])}"
            f"&redirect_uri={KEYCLOAK_REDIRECT_URI}"
        )

        return redirect(redirect_url)
    msg = None
    if request.method == "POST":
        login_form = LoginForm(request.POST)
        if login_form.is_valid():
            username = login_form.cleaned_data.get("username")
            password = login_form.cleaned_data.get("password")
            user = authenticate(username=username, password=password)
            if user is not None:
                if request.user:
                    logout(request)
                login(request, user)

                logger.info("Очистка устаревших сессий...")
                clear_previos_sessions(request)

                next = None
                if CACHE_CALLBACK:
                    next = CACHE_CALLBACK(request)

                logger.info("Пользователь %s авторизован", username)
                return redirect(request.GET.get("next", next or DEFAULT_REDIRECT_URI))
            logger.warning("Пользователь %s ввёл неверные учётные данные", username)
            msg = "Неправильный логин или пароль"
        else:
            logger.info("Форма авторизации не прошла проверку")
            msg = "Некорректный ввод"
    else:
        user = request.user
        if user and user.is_authenticated:
            logger.info("Пользователь %s уже авторизован.", user.get_username())
            return redirect(request.GET.get("next", DEFAULT_REDIRECT_URI))

    form = LoginForm(request.POST or None)
    return render(
        request,
        AUTH_PAGE_DOCUMENT or "templates/login.html",
        context={
            "app_name": "authentication",
            "title": "Вход",
            "center": True,
            "msg": msg,
            "version": VERSION_TAG,
            "form": form,
        },
    )


def _get_code(request: HttpRequest) -> str:
    code = request.GET.get("code")
    if not code:
        msg = "Не предоставлен code в обратном вызове Keycloak"  # noqa: RUF001
        raise KeyError(msg)
    return code


def keycloak_callback(request: HttpRequest) -> HttpResponse | HttpResponseRedirect | HttpResponsePermanentRedirect:
    logger.info("Обратный вызов из Keycloak")
    try:
        code = _get_code(request)
    except Exception:
        return render(
            request,
            ERROR_PAGE_DOCUMENT or "templates/500.html",
            {
                "code": 500,
                "message": "Ошибка аутентификации в Keycloak, обратный вызов не вернул код, пожалуйста, \
                    обратитесь к администратору!",
                "header": "Ошибка аутентификации Keycloak",
                "button_text": "Вернуться к авторизации",
                "logout_url": reverse_lazy("yarik_django_auth:login"),
            },
        )
    logger.info("Авторизационный код получен")
    logger.debug("Код: %s", code)

    backend = KeycloakConfidentialBackend()
    logger.info("Обмен кода на токен")
    data_token = backend.exchange_code_for_token(code)

    if not data_token:
        logger.warning("Ошибка: токен не получен")
        return render(
            request,
            ERROR_PAGE_DOCUMENT or "templates/403.html",
            {
                "code": 500,
                "message": "Ошибка аутентификации в Keycloak, не удалось получить токен, пожалуйста, \
                    обратитесь к администратору!",
                "header": "Ошибка аутентификации Keycloak",
                "button_text": "Вернуться к авторизации",
                "logout_url": reverse_lazy("yarik_django_auth:login"),
            },
        )

    logger.info("Токен получен")
    logger.info("Начало аутентификации")

    try:
        user = backend.authenticate(data_token)
    except jwt.exceptions.ImmatureSignatureError:
        logger.exception(
            "Ошибка проверки подписи токена, токен предоставляет время в будущем, синхронизируйте системное время"
        )
        return render(
            request,
            ERROR_PAGE_DOCUMENT or "templates/500.html",
            {
                "code": 500,
                "message": "Токен не прошёл проверку, пожалуйста обратитесь к администратору!",
                "header": "Ошибка аутентификации в Keycloak",
                "button_text": "Выход",
                "logout_url": f"{KEYCLOAK_URL_BASE_CLIENT}realms/{REALM_NAME}/protocol/openid-connect/logout"
                f"?client_id={CLIENT_ID}&post_logout_redirect_uri={KEYCLOAK_POST_LOGOUT_URL}",
            },
        )
    except PermissionError:
        logger.exception("У клиента нет доступа к интерфейсу")  # noqa: RUF001
        return render(
            request,
            ERROR_PAGE_DOCUMENT or "templates/403.html",
            {
                "code": 403,
                "message": "У данного пользователя нет доступа к интерфейсу, пожалуйста обратитесь к администратору!",  # noqa: RUF001
                "header": "Ошибка аутентификации в Keycloak",
                "button_text": "Выход",
                "logout_url": f"{KEYCLOAK_URL_BASE_CLIENT}realms/{REALM_NAME}/protocol/openid-connect/logout"
                f"?client_id={CLIENT_ID}&post_logout_redirect_uri={KEYCLOAK_POST_LOGOUT_URL}",
            },
        )
    except Exception as e:
        logger.exception("Неизвестная ошибка при декодировании токена")
        logger.debug(e.args)
        return render(
            request,
            ERROR_PAGE_DOCUMENT or "templates/520.html",
            {
                "code": 520,
                "message": "Неизвестная ошибка, пожалуйста обратитесь к администратору!",
                "header": "Ошибка аутентификации в Keycloak",
                "button_text": "Выход",
                "logout_url": f"{KEYCLOAK_URL_BASE_CLIENT}realms/{REALM_NAME}/protocol/openid-connect/logout"
                f"?client_id={CLIENT_ID}&post_logout_redirect_uri={KEYCLOAK_POST_LOGOUT_URL}",
            },
        )

    if user is not None:
        login(request, user, backend="django.contrib.auth.backends.ModelBackend")

        logger.info("Очистка устаревших сессий...")
        clear_previos_sessions(request)
        next = None
        if CACHE_CALLBACK:
            next = CACHE_CALLBACK(request)

        logger.info("Пользователь %s авторизован.", user.username)
        request.session["oidc_auth_token"] = data_token
        return redirect(request.GET.get("next", next or DEFAULT_REDIRECT_URI))
    return render(
        request,
        ERROR_PAGE_DOCUMENT or "templates/404.html",
        {
            "code": 404,
            "message": "Аутентификация в Keycloak прошла, однако данного пользователя не существует в системе, \
                пожалуйста обратитесь к администратору!",
            "header": "Ошибка авторизации",
            "button_text": "Выход",
            "logout_url": f"{KEYCLOAK_URL_BASE_CLIENT}realms/{REALM_NAME}/protocol/openid-connect/logout"
            f"?client_id={CLIENT_ID}&post_logout_redirect_uri={KEYCLOAK_POST_LOGOUT_URL}",
        },
    )


@login_required(login_url=reverse_lazy("yarik_django_auth:login"))
def logout_view(request: HttpRequest) -> HttpResponseRedirect | HttpResponsePermanentRedirect:
    user = request.user
    if CLEAR_CACHE_CALLBACK and isinstance(user, User):
        CLEAR_CACHE_CALLBACK(user)
    logout(request)
    logger.info("Пользователь %s вышел из системы", user.get_username())
    if USE_KEYCLOAK:
        redirect_url = (
            f"{KEYCLOAK_URL_BASE_CLIENT}realms/{REALM_NAME}/protocol/openid-connect/logout"
            f"?client_id={CLIENT_ID}&post_logout_redirect_uri={KEYCLOAK_POST_LOGOUT_URL}"
        )

        return redirect(redirect_url)
    return redirect(reverse_lazy("yarik_django_auth:login"))


@ajax_login_required
def get_new_csrftoken(request: HttpRequest) -> JsonResponse:
    token = get_token(request)
    return JsonResponse({"token": token})



import logging
from collections.abc import Callable
from functools import wraps
from typing import Concatenate

from django.contrib.auth.models import User
from django.contrib.sessions.models import Session
from django.http import HttpRequest, JsonResponse

logger = logging.getLogger(__name__)


def clear_previos_sessions(request: HttpRequest) -> None:
    sessions = Session.objects.order_by("-expire_date")
    if not isinstance(request.user, User):
        msg = "Пользователь не авторизован"
        raise TypeError(msg)
    for session in sessions:
        session_data_decoded = session.get_decoded()
        if "_auth_user_id" in session_data_decoded and int(session_data_decoded["_auth_user_id"]) == int(
            request.user.pk
        ):
            session.delete()


def ajax_login_required[**P, R](
    view_func: Callable[Concatenate[HttpRequest, P], R],
) -> Callable[Concatenate[HttpRequest, P], R | JsonResponse]:
    @wraps(view_func)
    def wrapper(request: HttpRequest, *args: P.args, **kwargs: P.kwargs) -> R | JsonResponse:
        if not request.user.is_authenticated:
            logger.warning("Попытка неавторизованного доступа: запрос отклонён.", request.user)
            return JsonResponse({"msg": "Unauthorized"}, status=401)
        return view_func(request, *args, **kwargs)

    return wrapper



from collections.abc import Callable
from importlib import import_module

from django.conf import settings
from django.contrib.auth.models import User
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpRequest

USE_KEYCLOAK = getattr(settings, "USE_KEYCLOAK", False)


KEYCLOAK_POST_LOGOUT_URL = getattr(settings, "KEYCLOAK_POST_LOGOUT_URL", None)
REALM_NAME = getattr(settings, "REALM_NAME", None)
CLIENT_ID = getattr(settings, "CLIENT_ID", None)
KEYCLOAK_URL_BASE_CLIENT = getattr(settings, "KEYCLOAK_URL_BASE_CLIENT", None)
KEYCLOAK_SCOPES = getattr(settings, "KEYCLOAK_SCOPES", None)
KEYCLOAK_REDIRECT_URI = getattr(settings, "KEYCLOAK_REDIRECT_URI", None)
CLIENT_SECRET = getattr(settings, "CLIENT_SECRET", None)
KEYCLOAK_URL_BASE = getattr(settings, "KEYCLOAK_URL_BASE", None)
KEYCLOAK_AUDIENCE = getattr(settings, "KEYCLOAK_AUDIENCE", None)
KEYCLOAK_IS_CREATE = getattr(settings, "KEYCLOAK_IS_CREATE", True)
KEYCLOAK_CERT_FILENAME = getattr(settings, "KEYCLOAK_CERT_FILENAME", "/home/sdl/cert/host.pem")
AUTH_PAGE_DOCUMENT = getattr(settings, "AUTH_PAGE_DOCUMENT", None)
ERROR_PAGE_DOCUMENT = getattr(settings, "ERROR_PAGE_DOCUMENT", None)

VERSION_TAG = getattr(settings, "VERSION_TAG", None)

DEFAULT_REDIRECT_URI = getattr(settings, "DEFAULT_REDIRECT_URI", "/")

DEFAULT_REQUEST_TIMEOUT = 60


def get_cache_callback() -> Callable[[HttpRequest], str] | None:
    path = getattr(settings, "CACHE_CALLBACK", None)
    if path is None:
        return None
    module_name, func_name = path.rsplit(".", 1)
    module = import_module(module_name)
    return getattr(module, func_name)


def get_clear_cache_callback() -> Callable[[User], None] | None:
    path = getattr(settings, "CLEAR_CACHE_CALLBACK", None)
    if path is None:
        return None
    module_name, func_name = path.rsplit(".", 1)
    module = import_module(module_name)
    return getattr(module, func_name)


CACHE_CALLBACK = get_cache_callback()
CLEAR_CACHE_CALLBACK = get_clear_cache_callback()


if USE_KEYCLOAK and not (
    CLIENT_ID is not None
    and CLIENT_SECRET is not None
    and REALM_NAME is not None
    and KEYCLOAK_URL_BASE is not None
    and KEYCLOAK_POST_LOGOUT_URL is not None
    and KEYCLOAK_REDIRECT_URI is not None
    and KEYCLOAK_IS_CREATE is not None
):
    msg = """Не удалось найти параметры KEYCLOAK_CLIENT_ID, KEYCLOAK_CLIENT_SECRET, KEYCLOAK_REALM_NAME, \
        KEYCLOAK_URL_BASE, KEYCLOAK_POST_LOGOUT_URL в настройках приложения"""  # noqa: RUF001
    raise ImproperlyConfigured(msg)



import datetime
import logging
from collections.abc import Awaitable, Callable
from typing import Any
from urllib.parse import quote, urlencode

import jwt
import requests
from asgiref.sync import iscoroutinefunction, markcoroutinefunction, sync_to_async
from django.contrib.auth import logout
from django.http import (
    HttpRequest,
    HttpResponse,
    JsonResponse,
)
from django.shortcuts import render
from django.urls import reverse_lazy

from yarik_django_auth.conf.settings import (
    CLIENT_ID,
    CLIENT_SECRET,
    DEFAULT_REQUEST_TIMEOUT,
    ERROR_PAGE_DOCUMENT,
    KEYCLOAK_CERT_FILENAME,
    KEYCLOAK_SCOPES,
    KEYCLOAK_URL_BASE,
    REALM_NAME,
)

logger = logging.getLogger(__name__)


def check_session(access_token: str) -> int:
    resp_status = 401
    try:
        userinfo_endpoint = f"{KEYCLOAK_URL_BASE}realms/{REALM_NAME}/protocol/openid-connect/userinfo"
        response = requests.get(
            userinfo_endpoint,
            headers={"Authorization": f"Bearer {access_token}"},
            verify=KEYCLOAK_CERT_FILENAME,
            timeout=DEFAULT_REQUEST_TIMEOUT,
        )
        logger.debug("Выполнен запрос проверки актуальности сессии")
        resp_status = response.status_code
    except Exception as e:
        logger.debug(e)

    return resp_status


def update_token(refresh_token: str) -> tuple[Any | None, int]:
    resp_status = 401
    data = None
    try:
        token_endpoint = f"{KEYCLOAK_URL_BASE}realms/{REALM_NAME}/protocol/openid-connect/token"
        payload: dict[str, str | None] = {
            "client_id": CLIENT_ID,
            "grant_type": "refresh_token",
            "refresh_token": refresh_token,
            "scope": " ".join(KEYCLOAK_SCOPES or []),
            "client_secret": CLIENT_SECRET,
        }
        response = requests.post(
            token_endpoint,
            headers={"Content-Type": "application/x-www-form-urlencoded"},
            verify=KEYCLOAK_CERT_FILENAME,
            data=urlencode(payload, quote_via=quote),
            timeout=DEFAULT_REQUEST_TIMEOUT,
        )
        logger.debug("Выполнен запрос обновления токена")
        logger.debug(response.text)

        resp_status = response.status_code
        if resp_status == 200:
            data = response.json()
    except Exception as e:
        logger.debug(e)

    return (data, resp_status)


class AsyncMiddleware:
    async_capable = True
    sync_capable = False

    def __init__(self, get_response: Callable[[HttpRequest], Awaitable[JsonResponse | HttpResponse]]) -> None:
        self.get_response = get_response
        if iscoroutinefunction(self.get_response):
            markcoroutinefunction(self)

    async def get_403_response(self, request: HttpRequest) -> JsonResponse:
        msg = "Ошибка при проверке актуальности сессии. Возможно токен не содержит scope openid."
        logger.error(msg)
        logout(request)
        return JsonResponse({"msg": msg}, status=403)

    async def __call__(self, request: HttpRequest) -> JsonResponse | HttpResponse:
        if "oidc_auth_token" in request.session:
            oidc_auth_token = request.session["oidc_auth_token"]
            access_token = oidc_auth_token.get("access_token")
            refresh_token = oidc_auth_token.get("refresh_token")
            logger.debug("Токен извлеченный из сессии = %s", oidc_auth_token)
            logger.debug("Токен доступа = %s", access_token)

            def _get_user(r: HttpRequest) -> str:
                return r.user.username

            username = await sync_to_async(_get_user)(request)

            # 1. Проверяем актуальность токена
            logger.debug("Проверка актуальности токена доступа")
            decoded_token = jwt.decode(access_token, verify=False, options={"verify_signature": False})
            if datetime.datetime.fromtimestamp(decoded_token["exp"], tz=datetime.UTC) < datetime.datetime.now(
                tz=datetime.UTC
            ):
                # Токен просрочен, надо обновить
                logger.debug("Токен устарел: обновление токена...")

                data_token, resp_status = update_token(refresh_token)

                logger.debug("Код ответа: %s", resp_status)

                if not data_token:
                    # Токен не получен, очистка сессии
                    logger.info("Сессия очищена для пользователя %s. Причина: не удалось обновить токен", username)
                    await sync_to_async(logout)(request)
                    return await self.get_response(request)

                logger.debug("Сохранение токена в сессии...")
                request.session["oidc_auth_token"] = data_token

                access_token = data_token["access_token"]

            # 2. Проверяем актуальность сессии
            logger.debug("Выполнение запроса проверки актуальности сессии")

            resp_status = check_session(access_token)

            logger.debug("Код ответа: %s", resp_status)

            if resp_status == 403:
                # В запросе не хватает данных  # noqa: RUF003
                return await self.get_403_response(request)

            if resp_status != 200:
                # Сессия просрочена или refresh_token устарел, очистка сессии
                logger.info(
                    "Сессия очищена для пользователя %s. Причина: сессия в Keycloak была завершена",
                    username,
                )
                await sync_to_async(logout)(request)

        return await self.get_response(request)


class SyncMiddleware:
    async_capable = False
    sync_capable = True

    def __init__(self, get_response: Callable[[HttpRequest], JsonResponse | HttpResponse]) -> None:
        self.get_response = get_response

    def get_403_response(self, request: HttpRequest) -> HttpResponse:
        logger.error("Ошибка при проверке актуальности сессии. Возможно токен не содержит scope openid.")
        logout(request)
        return render(
            request,
            ERROR_PAGE_DOCUMENT or "templates/403.html",
            {
                "code": 403,
                "message": "Ошибка аутентификации в Keycloak, при проверке актуальности сессии произошла ошибка, \
                    обратитесь к администратору!",
                "header": "Ошибка аутентификации Keycloack",
                "button_text": "Вернуться к авторизации",
                "logout_url": reverse_lazy("yarik_django_auth:logout"),
            },
        )

    def __call__(self, request: HttpRequest) -> JsonResponse | HttpResponse:
        if "oidc_auth_token" in request.session:
            oidc_auth_token = request.session["oidc_auth_token"]
            access_token = oidc_auth_token.get("access_token")
            refresh_token = oidc_auth_token.get("refresh_token")
            logger.debug("Токен извлеченный из сессии = %s", oidc_auth_token)
            logger.debug("Токен доступа = %s", access_token)

            # 1. Проверяем актуальность токена
            logger.debug("Проверка актуальности токена доступа")
            decoded_token = jwt.decode(access_token, verify=False, options={"verify_signature": False})
            if datetime.datetime.fromtimestamp(decoded_token["exp"], tz=datetime.UTC) < datetime.datetime.now(
                tz=datetime.UTC
            ):
                # Токен просрочен, надо обновить
                logger.debug("Токен устарел: обновление токена...")

                data_token, resp_status = update_token(refresh_token)

                logger.debug("Код ответа: %s", resp_status)

                username = request.user.username

                if not data_token:
                    # Токен не получен, очистка сессии
                    logger.info("Сессия очищена для пользователя %s. Причина: не удалось обновить токен", username)
                    logout(request)
                    return self.get_response(request)

                logger.debug("Сохранение токена в сессии...")
                request.session["oidc_auth_token"] = data_token

                access_token = data_token["access_token"]

            # 2. Проверяем актуальность сессии
            logger.debug("Выполнение запроса проверки актуальности сессии")

            resp_status = check_session(access_token)

            logger.debug("Код ответа: %s", resp_status)

            if resp_status == 403:
                # В запросе не хватает данных  # noqa: RUF003
                return self.get_403_response(request)

            if resp_status != 200:
                # Сессия просрочена или refresh_token устарел, очистка сессии
                logger.info(
                    "Сессия очищена для пользователя %s. Причина: сессия в Keycloak была завершена",
                    request.user.username,
                )
                logout(request)

        return self.get_response(request)



"""Представления пакета авторизации."""

from django.http import HttpRequest, HttpResponse, HttpResponsePermanentRedirect, HttpResponseRedirect, JsonResponse

def index(_: HttpRequest) -> HttpResponseRedirect | HttpResponsePermanentRedirect: ...
def login_view(request: HttpRequest) -> HttpResponseRedirect | HttpResponsePermanentRedirect | HttpResponse: ...
def keycloak_callback(request: HttpRequest) -> HttpResponse | HttpResponseRedirect | HttpResponsePermanentRedirect: ...
def logout_view(request: HttpRequest) -> HttpResponseRedirect | HttpResponsePermanentRedirect: ...
def get_new_csrftoken(request: HttpRequest) -> JsonResponse: ...
